==> libunistring-1.0.tar.xz: libunistring-1.0/lib/unistr/u16-strcmp.c <==
/* Compare UTF-16 strings.
   Copyright (C) 1999, 2002, 2006, 2009-2022 Free Software Foundation, Inc.
   Written by Bruno Haible <bruno@clisp.org>, 2002.

   This file is free software.
   It is dual-licensed under "the GNU LGPLv3+ or the GNU GPLv2+".
   You can redistribute it and/or modify it under either
     - the terms of the GNU Lesser General Public License as published
       by the Free Software Foundation; either version 3, or (at your
       option) any later version, or
     - the terms of the GNU General Public License as published by the
       Free Software Foundation; either version 2, or (at your option)
       any later version, or
     - the same dual license "the GNU LGPLv3+ or the GNU GPLv2+".

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License and the GNU General Public License
   for more details.

   You should have received a copy of the GNU Lesser General Public
   License and of the GNU General Public License along with this
   program.  If not, see <https://www.gnu.org/licenses/>.  */

#include <config.h>

/* Specification.  */
#include "unistr.h"

int
u16_strcmp (const uint16_t *s1, const uint16_t *s2)
{
  /* Note that the UTF-16 encoding does NOT preserve lexicographic order.
     Namely, if uc1 is a 16-bit character and [uc2a,uc2b] is a surrogate pair,
     we must enforce uc1 < [uc2a,uc2b], even if uc1 > uc2a.  */
  for (;;)
    {
      uint16_t c1 = *s1++;
      uint16_t c2 = *s2++;
      if (c1 != 0 && c1 == c2)
        continue;
      if (c1 < 0xd800 || c1 >= 0xe000)
        {
          if (!(c2 < 0xd800 || c2 >= 0xe000))
            /* c2 is a surrogate, but c1 is not.  */
            return -1;
        }
      else
        {
          if (c2 < 0xd800 || c2 >= 0xe000)
            /* c1 is a surrogate, but c2 is not.  */
            return 1;
        }
      return (int)c1 - (int)c2;
      /* > 0 if c1 > c2, < 0 if c1 < c2. */
    }
}

==> libunistring-1.1.tar.xz: libunistring-1.1/lib/unistr/u16-strcmp.c <==
/* Compare UTF-16 strings.
   Copyright (C) 1999, 2002, 2006, 2009-2022 Free Software Foundation, Inc.
   Written by Bruno Haible <bruno@clisp.org>, 2002.

   This file is free software.
   It is dual-licensed under "the GNU LGPLv3+ or the GNU GPLv2+".
   You can redistribute it and/or modify it under either
     - the terms of the GNU Lesser General Public License as published
       by the Free Software Foundation, either version 3, or (at your
       option) any later version, or
     - the terms of the GNU General Public License as published by the
       Free Software Foundation; either version 2, or (at your option)
       any later version, or
     - the same dual license "the GNU LGPLv3+ or the GNU GPLv2+".

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License and the GNU General Public License
   for more details.

   You should have received a copy of the GNU Lesser General Public
   License and of the GNU General Public License along with this
   program.  If not, see <https://www.gnu.org/licenses/>.  */

#include <config.h>

/* Specification.  */
#include "unistr.h"

int
u16_strcmp (const uint16_t *s1, const uint16_t *s2)
{
  /* Note that the UTF-16 encoding does NOT preserve lexicographic order.
     Namely, if uc1 is a 16-bit character and [uc2a,uc2b] is a surrogate pair,
     we must enforce uc1 < [uc2a,uc2b], even if uc1 > uc2a.  */
  for (;;)
    {
      uint16_t c1 = *s1++;
      uint16_t c2 = *s2++;
      if (c1 != 0 && c1 == c2)
        continue;
      if (c1 < 0xd800 || c1 >= 0xe000)
        {
          if (!(c2 < 0xd800 || c2 >= 0xe000))
            /* c2 is a surrogate, but c1 is not.  */
            return -1;
        }
      else
        {
          if (c2 < 0xd800 || c2 >= 0xe000)
            /* c1 is a surrogate, but c2 is not.  */
            return 1;
        }
      return (int)c1 - (int)c2;
      /* > 0 if c1 > c2, < 0 if c1 < c2. */
    }
}
